﻿using System;
using System.Threading;
using System.Threading.Tasks;

namespace TestProject
{
    /// <summary>
    /// 프로그램
    /// </summary>
    class Program
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region 프로그램 시작하기 - Main(argumentArray)

        /// <summary>
        /// 프로그램 시작하기
        /// </summary>
        /// <param name="argumentArray">인자 배열</param>
        private static void Main(string[] argumentArray)
        {
            Action action = () =>
            {
                Console.WriteLine(DateTime.Now.ToString("HH:mm:ss"));
            };

            StartPeriodicTask(action, 1000, 0, CancellationToken.None);

            Console.ReadKey(false);
        }

        #endregion
        #region 일정 주기 반복 태스크 시작하기 - StartPeriodicTask(action, interval, delay, token)

        /// <summary>
        /// 일정 주기 반복 태스크 시작하기
        /// </summary>
        /// <param name="action"></param>
        /// <param name="interval">간격 (단위 : 밀리초)</param>
        /// <param name="delay">지연 (단위 : 밀리초)</param>
        /// <param name="token">취소 토큰</param>
        /// <returns>태스크</returns>
        private static Task StartPeriodicTask(Action action, int interval, int delay, CancellationToken token)
        {
            Action wrapperAction = () =>
            {
                if(token.IsCancellationRequested)
                {
                    return;
                } 

                action();
            }; 

            Action mainAction = () =>
            {
                TaskCreationOptions options = TaskCreationOptions.AttachedToParent; 

                if(token.IsCancellationRequested)
                {
                    return;
                } 

                if(delay > 0)
                {
                    Thread.Sleep(delay);
                } 

                while(true)
                {
                    if(token.IsCancellationRequested)
                    {
                        break;
                    } 

                    Task.Factory.StartNew(wrapperAction, token, options, TaskScheduler.Current); 

                    if(token.IsCancellationRequested || interval == Timeout.Infinite)
                    {
                        break;
                    } 

                    Thread.Sleep(interval);
                }
            }; 

            return Task.Factory.StartNew(mainAction, token);
        }

        #endregion
    }
}